#include <stdio.h>
#include <stdlib.h>

#include <netcdf.h>

static int ncstatus;
#define handle_error(a) if ((ncstatus = (a)) != NC_NOERR) { \
    fprintf(stderr, "NetCDF error on line %d: %s\n", __LINE__, \
      nc_strerror(ncstatus)); nc_close(ncid); exit(-1); \
} 
#define report_error(a, s) if ((ncstatus = (a)) != NC_NOERR) { \
    fprintf(stderr, "NetCDF error on line %d: %s (%s)\n", \
	    __LINE__, nc_strerror(ncstatus), s); exit(-1); \
} 

static
char *
get_char_attribute(int ncid, int varid, char *name)
{
  size_t length;
  nc_type type;
  char *value;

  report_error(nc_inq_atttype(ncid, varid, name, &type), name);
  if (type != NC_CHAR) {
    fprintf(stderr, "Error: attribute %s is not of type char\n", name);
    exit(-1);
  }
  report_error(nc_inq_attlen(ncid, varid, name, &length), name);
  value = malloc((length+1) * sizeof(char));
  if (!value) {
    fprintf(stderr, "Out of memory\n");
    exit(2);
  }
  report_error(nc_get_att_text(ncid, varid, name, value), name);
  value[length] = '\0';
  return value;
}

static
short
get_short_attribute(int ncid, int varid, char *name)
{
  size_t length;
  short value;

  if (nc_inq_attlen(ncid, varid, name, &length) != NC_NOERR) {
    fprintf(stderr, "Error: %s attribute is not of type short\n", name);
    exit(1);
  }
  if (length > 1) {
    fprintf(stderr, "Error: %s attribute has more than 1 element\n", name);
    exit(1);
  }
  handle_error(nc_get_att_short(ncid, varid, name, &value));
  return value;
}

static
void
Usage(char **argv)
{
  fprintf(stderr, "Usage: %s file1.nc [file2.nc ...] > infofile.txt \n", argv[0]);
}

int
main(int argc, char **argv)
{
  int iarg, ifile;

  /* Parse switches */
  iarg=1;
  while (iarg < argc && *argv[iarg] == '-') {
    iarg++;
  }

  if (iarg >= argc) {
    Usage(argv);
    exit(0);
  }

  printf("%%FileRastScan|  Date  | Time |Duration| Elevation |  Azimuth  |   Range   |Scan\n");
  printf("%%            |YYYYMMDD|HHMMSS|seconds |start   end|start   end|start   end|type\n");

  for (ifile = iarg; ifile < argc; ++ifile) {
    size_t ngates, nrays;
    size_t index;
    float duration;
    float start_elev, end_elev;
    float start_azim, end_azim;
    float start_time, end_time;
    float start_range, end_range;
    char *scan_type;

    int ncid;
    int timedimid, rangedimid;
    int elevid, azimid, rangeid, timeid;

    short file, raster, scan;
    short year, month, day;
    short hour, minute, second;

    handle_error(nc_open(argv[ifile], NC_NOWRITE, &ncid));
    handle_error(nc_inq_dimid(ncid, "time", &timedimid));
    handle_error(nc_inq_dimlen(ncid, timedimid, &nrays));
    handle_error(nc_inq_dimid(ncid, "range", &rangedimid));
    handle_error(nc_inq_dimlen(ncid, rangedimid, &ngates));

    /* Accept tape or file attributes */
    if (nc_inq_attlen(ncid, NC_GLOBAL, "tape", &index) == NC_NOERR) {
      file = get_short_attribute(ncid, NC_GLOBAL, "tape");
    }
    else {
      file = get_short_attribute(ncid, NC_GLOBAL, "file");
    }
    raster = get_short_attribute(ncid, NC_GLOBAL, "raster");
    scan = get_short_attribute(ncid, NC_GLOBAL, "scan");
    
    year = get_short_attribute(ncid, NC_GLOBAL, "year");
    month = get_short_attribute(ncid, NC_GLOBAL, "month");
    day = get_short_attribute(ncid, NC_GLOBAL, "day");

    scan_type = get_char_attribute(ncid, NC_GLOBAL, "scantype");

    handle_error(nc_inq_varid(ncid, "elev", &elevid));
    handle_error(nc_inq_varid(ncid, "azim", &azimid));
    handle_error(nc_inq_varid(ncid, "time", &timeid));
    handle_error(nc_inq_varid(ncid, "range", &rangeid));

    index = 0;
    handle_error(nc_get_var1_float(ncid, elevid, &index, &start_elev));
    handle_error(nc_get_var1_float(ncid, azimid, &index, &start_azim));
    handle_error(nc_get_var1_float(ncid, timeid, &index, &start_time));
    handle_error(nc_get_var1_float(ncid, rangeid, &index, &start_range));

    index = nrays-1;
    handle_error(nc_get_var1_float(ncid, elevid, &index, &end_elev));
    handle_error(nc_get_var1_float(ncid, azimid, &index, &end_azim));
    handle_error(nc_get_var1_float(ncid, timeid, &index, &end_time));

    index = ngates-1;
    handle_error(nc_get_var1_float(ncid, rangeid, &index, &end_range));

    hour = (short) start_time;
    minute = (short) ((start_time - hour) * 60.0);
    second = (short) ((start_time - hour - minute/60.0) * 3600.0 + 0.5);

    duration = (end_time-start_time) * 3600.0;
    if (duration < 0.0) {
      duration += 24.0;
    }

    printf(" %04d %03d %03d %04d%02d%02d %02d%02d%02d %8.1f"
	   " %5.1f %5.1f %5.1f %5.1f %5.1f %5.1f %s\n",
	   file, raster, scan,
	   year, month, day,
	   hour, minute, second, duration,
	   start_elev, end_elev,
	   start_azim, end_azim,
	   start_range, end_range, scan_type);

    free(scan_type);
    nc_close(ncid);
  }
  exit(0);
}
